#include "World.h"
#include "Ogre.h"
#include "OgreMath.h"
#include "OgreSceneManager.h"
#include "OgreSceneNode.h"
#include "OgreVector3.h"
#include "OgreSphere.h"
#include "MovingObject.h"
#include "InputHandler.h"
#include "CollisionManager.h"
#include "OgreAxisAlignedBox.h"
#include "TriggerArea.h"
#include "LuaWrapper.h"


World::World(Ogre::SceneManager *sceneManager, InputHandler *ih) : mSceneManager(sceneManager), mInputHandler(ih)
{

	mSceneManager->setAmbientLight(Ogre::ColourValue(1,1,1));

	MovingObject *o = createStaticObject("TankGroundPlane.mesh", "groundPlane");

	OBB *collision = o->getOBB();

	Ogre::AxisAlignedBox aabb = collision->getAABB();

	aabb.setMinimumY(-30);
	collision->resetAABB(aabb);

	mPlayer = createMovingObject("MyCube.mesh", "player");

}




void World::setScale(std::string objectname, Ogre::Vector3 newScale)
{
	mMovingObjects[objectname]->setScale(newScale);
}



MovingObject * World::createStaticObject(std::string meshName, std::string objectName)
{
	MovingObject *obj = new MovingObject(meshName, "static",  mSceneManager);
	mStaticObjects[std::string(objectName)] = obj;
	return obj;

}


MovingObject *World::getMovingObject(std::string name)
{
	if (mMovingObjects.find(name) == mMovingObjects.end())
	{
		return NULL;
	}
	return mMovingObjects[name];

}

TriggerArea *World::getTriggerArea(std::string name)
{
	if (mTriggerAreas.find(name) == mTriggerAreas.end())
	{
		return NULL;
	}
	return mTriggerAreas[name];

}


MovingObject * 
	World::createMovingObject(std::string meshName, std::string nodeName)
{
	if (mMovingObjects.find(nodeName) == mMovingObjects.end())
	{
		MovingObject *obj = new MovingObject(meshName, nodeName, mSceneManager);
		mMovingObjects[std::string(nodeName)] = obj;
		return obj;
	}
	else
	{
		return NULL;
	}
}


bool World::removeMovingObject( std::string nodeName)
{
	std::map<std::string, MovingObject *>::iterator it = mMovingObjects.find(nodeName);
	if (it== mMovingObjects.end())
	{
		return false;
	}
	else
	{
		MovingObject *obj  = it->second;
		mSceneManager->getRootSceneNode()->removeChild(obj->getSceneNode());
		delete(obj);
		mMovingObjects.erase(nodeName);
		return true;
	}
}




Ogre::Vector3 World::getPosition(std::string objectName)
{
	return mMovingObjects[objectName]->getPosition();
}


void
	World::setPosition(std::string nodename, Ogre::Vector3 newPosition)
{
	mMovingObjects[nodename]->setPosition(newPosition);
}

void
	World::setVelocity(std::string nodename, Ogre::Vector3 velocity)
{
	mMovingObjects[nodename]->setVelocity(velocity);
}

void World::handlePlayerInput(float time)
{
	float multiplier = 1;
	if (mInputHandler->isKeyDown(OIS::KC_SPACE))
	{
		multiplier = 10;
	}

	if (mInputHandler->isKeyDown(OIS::KC_UP))
	{
		mPlayer->setVelocity(mPlayer->getFacing() * 10 * multiplier);
	}
	else if (mInputHandler->isKeyDown(OIS::KC_DOWN))
	{
		mPlayer->setVelocity(mPlayer->getFacing() * -10 * multiplier);

	}
	else
	{
		mPlayer->setVelocity(Ogre::Vector3::ZERO);
	}
	if (mInputHandler->isKeyDown(OIS::KC_LEFT))
	{
		mPlayer->yaw(Ogre::Degree(time * 40));
	}
	if (mInputHandler->isKeyDown(OIS::KC_RIGHT))
	{
		mPlayer->yaw(Ogre::Degree(time * -40));
	}
	if (mInputHandler->isKeyDown(OIS::KC_A))
	{
		mPlayer->roll(Ogre::Degree(time * 40));
	}
	if (mInputHandler->isKeyDown(OIS::KC_D))
	{
		mPlayer->roll(Ogre::Degree(time * -40));
	}
	if (mInputHandler->isKeyDown(OIS::KC_W))
	{
		mPlayer->pitch(Ogre::Degree(time * 40));
	}
	if (mInputHandler->isKeyDown(OIS::KC_S))
	{
		mPlayer->pitch(Ogre::Degree(time * -40));
	}



}




TriggerArea * World::addTriggerArea(std::string name, Ogre::Vector3 location, Ogre::Vector3 size)
{
	if (mTriggerAreas.find(name) == mTriggerAreas.end())
	{
		TriggerArea *triggerArea = new TriggerArea(location, size, mSceneManager);
		mTriggerAreas[name] = triggerArea;
		return triggerArea;
	}
	else
	{
		return NULL;
	}

}


TriggerArea * World::addTriggerArea(std::string name)
{
	return addTriggerArea(name, Ogre::Vector3::ZERO, Ogre::Vector3::UNIT_SCALE);


}

bool  World::setTriggerAreaOnEnter(std::string name, std::string luaScrpt)
{

	if (mTriggerAreas.find(name) == mTriggerAreas.end())
	{
		return false;
	}
	else
	{
		mTriggerAreas[name]->setOnEnterCallback(luaScrpt);
		return true;
	}

}
bool  World::setTriggerAreaOnExit(std::string name, std::string luaScrpt) 
{
	if (mTriggerAreas.find(name) == mTriggerAreas.end())
	{
		return false;
	}
	else
	{
		mTriggerAreas[name]->setOnExitCallback(luaScrpt);
		return true;
	}

}

bool  World::setTriggerAreaLocation(std::string name, Ogre::Vector3 location)
{
	if (mTriggerAreas.find(name) == mTriggerAreas.end())
	{
		return false;
	}
	else
	{
		mTriggerAreas[name]->setPosition(location);
		return true;
	}

}
bool  World::setTriggerAreaSize(std::string name, Ogre::Vector3 size)
{
		if (mTriggerAreas.find(name) == mTriggerAreas.end())
	{
		return false;
	}
	else
	{
		mTriggerAreas[name]->setSize(size);
		return true;
	}

}

bool  World::setTriggerAreaVisible(std::string name, bool isVisible)
{
			if (mTriggerAreas.find(name) == mTriggerAreas.end())
	{
		return false;
	}
	else
	{
		mTriggerAreas[name]->setVisible(isVisible);
		return true;
	}

}




void 
	World::Think(float time)
{
	handlePlayerInput(time);

	for (std::map<std::string, MovingObject *>::iterator it = mMovingObjects.begin(); it != mMovingObjects.end(); it++)
	{
		std::string name = it->first;
		MovingObject *obj = it->second;
		if (true || obj->getVelocity() != Ogre::Vector3::ZERO)
		{
			Ogre::Vector3 pos = obj->getPosition();
			pos = pos + obj->getVelocity() * time;
			obj->setPosition(pos);
			for (std::map<std::string, MovingObject *>::iterator it2 = mMovingObjects.begin(); it2 != mMovingObjects.end(); it2++)
			{
				if (it2->second != obj)
				{
					Ogre::Vector3 MTD;
					if (obj->getOBB()->collides(it2->second->getOBB(), MTD))
					{
						obj->translate(MTD);
					}
				}
			}
			for (std::map<std::string, MovingObject *>::iterator it2 = mStaticObjects.begin(); it2 != mStaticObjects.end(); it2++)
			{
				if (it2->second != obj)
				{
					Ogre::Vector3 MTD = Ogre::Vector3::ZERO;
					MovingObject *colliderObj = it2->second;

					OBB *collider = it2->second->getOBB();
					OBB *mover = obj->getOBB();
					if  (mover->collides(collider, MTD))
					{
						obj->translate(MTD);
					}
				}
			}


			for (std::map<std::string, TriggerArea *>::iterator it2 = mTriggerAreas.begin(); it2 != mTriggerAreas.end(); it2++)
			{
				it2->second->checkObject(obj);
			}
		}

	}
}




